#################################################################################################
#Wolff et al. xxxxxxxxx
#Code for extracting information from data generated from ImageJ
#################################################################################################

library(tidyr)
library(dplyr)
library(sjmisc)


#DATA PREPARATION -----------------------------------------------------------------------------

    #get csv file names
        files <- list.files(path = "data_ImageJ", pattern = ".csv", full.names=TRUE)
    
    #create dataframe with information from file names
        df <- data.frame(files, stringsAsFactors = FALSE)
    
        #separate into columns
            df <- separate(df, "files", c("temp1", 
                                      "Spider_ID", 
                                      "video", 
                                      "orientation", 
                                      "scale", 
                                      "body_length", 
                                      "framerate",
                                      "temp2"), 
                                      sep="\\,|\\/|\\.|~")
    
        
        #remove unnecessary columns
            df <- df[,-c(1,8)]
    
    
#CALCULATE/EXTRACT VALUES FOR EACH RECORDING--------------------------------------------------

    #create new dataframe
      final_data <- data.frame(matrix(NA, nrow = length(files), ncol = 7))
      colnames(final_data) <- c("Spider_ID", "video", "mean_speed", "mean_speed_bl", "peak_speed", "peak_speed_bl", "body_length")
    
    #preparation
    for (j in 1:length(files)){
        data <- read.csv(files[j], header=TRUE) #read csv file
        scale <- as.numeric(df[j,4])*0.1/7.1 #convert to pixels/mm
        body <- as.numeric(df[j,5])/scale #calculate body length in mm
        frame <- as.numeric(df[j,6]) #extract framerate
        
        
    #calculate distance
        #create new column
            data$distance <- NA
        
        #calculate distance traveled between each point
            data$distance[1] <- 0
            
            for (i in 2:length(data$Len..pixel.)){
                data$distance[i] <- (data$Len..pixel.[i]-data$Len..pixel.[i-1])/scale
            }
         
        
    #calculate velocity
        #create new column
            data$velocity <- NA
        
        #calculate velocity for each point
            data$velocity[1] <- 0
            
            for (k in 2:length(data$distance)){
                data$velocity[k] <- (data$distance[k]/1000)/(1/frame)
            }
            
        
        #remove first and last row (not used)
            data <- data[-c(1, length(data$velocity)), ]
        

    #populate final spreadsheet
        #get values for smoothed line
            smooth_vel <- smooth.spline(data$velocity, nknots = length(data$velocity)/2+1)
        
        #calculate means and peaks and write to dataframe
            final_data$Spider_ID[j] <- df[j,1] #Spider ID
            final_data$video[j] <- df[j,2] #video name
            final_data$mean_speed[j] <- mean(data$velocity) #mean speed
            final_data$mean_speed_bl[j] <- mean(data$velocity)/(body*0.001) #mean speed relative to body length
            final_data$peak_speed[j] <- max(smooth_vel$y) #max speed
            final_data$peak_speed_bl[j] <- max(smooth_vel$y)/(body*0.001) #max speed relative to body length
            final_data$body_length[j] <- body #body length [mm]
    }

        
    #merge with species data
        #import species list
           specieslist <- read.csv("data_other/species_list.csv")
           
        #create unique species name
           specieslist$species <- paste(specieslist$Family, specieslist$Subfamily, specieslist$Genus, specieslist$Species, sep=' ')
           specieslist$species <- gsub("  ", " ", specieslist$species)
          
        #merge species name with data
           final_data <- merge(final_data, specieslist[,c('Spider_ID', 'species')], by="Spider_ID")
           
        #move column
           final_data <- final_data%>%
             move_columns("species", .after="Spider_ID")
        
        
    #write data to csv
        write.csv (final_data, "output/spider_speed.csv", row.names = FALSE)

        
#CALCULATE AVERAGE VALUES PER SPECIES -----------------------------------------------------------------------
        
    #select maximum speed of each individual from available track
        max_peak_speed <- final_data %>%
              select(c(Spider_ID, peak_speed))%>%
              group_by(Spider_ID) %>%
              filter(peak_speed == max(peak_speed))
    
    #select maximum speed relative to body length of each available track
        max_peak_speed_bl <- final_data %>%
              select(c(Spider_ID, peak_speed_bl))%>%
              group_by(Spider_ID) %>%
              filter(peak_speed_bl == max(peak_speed_bl))
    
    #calculate mean body size of each species
        body_length_avg <- aggregate(final_data$body_length, list(Spider_ID=final_data$Spider_ID), mean)
        colnames(body_length_avg)[2] <- "body_length"
        
    #merge
        max_speed <- merge(specieslist[,c('species', 'Spider_ID')], max_peak_speed, by='Spider_ID')%>%
          merge(max_peak_speed_bl, by="Spider_ID") %>%
          merge(body_length_avg, by="Spider_ID")
        
    #calculate speed and sd for each species
        max_speed_avg <- max_speed %>%
              group_by(species) %>%
              summarise(n = n(),peak_speed_avg = mean(peak_speed), peak_speed_avg_sd = sd(peak_speed), 
                        peak_speed_bl_avg = mean(peak_speed_bl), peak_speed_bl_age_sd = sd(peak_speed_bl), 
                        body_length_avg=mean(body_length), body_length_avg_sd=sd(body_length))
    
    #write to file
      write.csv(max_speed_avg, "output/spider_species_speed.csv", row.names = FALSE)
      

